#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup(){
    ofSetLogLevel(OF_LOG_VERBOSE);
    
    // enable depth->video image calibration
    kinect.setRegistration(true);
    
    kinect.init();
    kinect.open();		// opens first available kinect
    // print the intrinsic IR sensor values
    if(kinect.isConnected()) {
        ofLogNotice() << "sensor-emitter dist: " << kinect.getSensorEmitterDistance() << "cm";
        ofLogNotice() << "sensor-camera dist:  " << kinect.getSensorCameraDistance() << "cm";
        ofLogNotice() << "zero plane pixel size: " << kinect.getZeroPlanePixelSize() << "mm";
        ofLogNotice() << "zero plane dist: " << kinect.getZeroPlaneDistance() << "mm";
    }
    
    grayscaleImage.allocate(kinect.width, kinect.height);
    colorImg.allocate(kinect.width, kinect.height);
    depthImg.allocate(kinect.width, kinect.height);
    grayThreshNear.allocate(kinect.width, kinect.height);
    grayThreshFar.allocate(kinect.width, kinect.height);

    nearThreshold = 230;
    farThreshold = 80;

    ofSetFrameRate(20); // Limits the frames to 20 per second.
    
    // zero the tilt on startup
    angle = 0;
    kinect.setCameraTiltAngle(angle);
    
    //    testImgArray.resize(20);
    nrBlobs = 0;
    
    //Take picture

    
    // Initialize the blob detection
    threshold = 52;
    minBlobSize = 1000;
    maxBlobSize = (kinect.width*kinect.height)/2;
    nBlobsConsidered = 20;
    
    // Loading the Haar-Cascade Featurs
    //    faceFinder.setup("haarcascade_frontalface_default.xml");
    //    faceFinder.setup("haarcascade_frontalface_alt.xml");
    //    faceFInder.setup("haarcascade_frontalface_alt2.xml");
    faceFinder.setup("haarcascade_frontalface_alt3.xml");
    

    
}

//--------------------------------------------------------------
void ofApp::update(){
    
    ofBackground(100, 100, 100);
    
    kinect.update();
    
    // there is a new frame and we are connected
    if(kinect.isFrameNew()) {
        
        // load grayscale depth image from the kinect source
        depthImg.setFromPixels(kinect.getDepthPixels(), kinect.width, kinect.height);
        colorImg.setFromPixels(kinect.getPixels(), kinect.width, kinect.height);
        // converts a color image to a grayscale image
        grayscaleImage = colorImg;
        
        //two thresholds - one for the far plane and one for the near plane
        //cvAnd to get the pixels which are a union of the two thresholds
        
        grayThreshNear = depthImg;
        grayThreshFar = depthImg;
        grayThreshNear.threshold(nearThreshold, true);
        grayThreshFar.threshold(farThreshold);
        cvAnd(grayThreshNear.getCvImage(), grayThreshFar.getCvImage(), depthImg.getCvImage(), NULL);
        
        // update the cv images
        depthImg.flagImageChanged();
        
        // find contours which are between the minBlobSize and maxBlobSize
        contourFinder.findContours(depthImg, minBlobSize, maxBlobSize, nBlobsConsidered, false, true);
        
        
        // clears the vector holding all of the previus blob images
        blobImgArray.clear();
        // clears the vector all of the old information about the detected faces
        blobFaceArray.clear();
        // clears the vector that stored how many faces was detected in eatch blob
        blobNrFaces.clear();
        nrBlobs = 0;
        
        for (int i = 0; i < contourFinder.nBlobs; i++) {
            
            // Extracting the rectangel of the blob
            faceRectangle = contourFinder.blobs.at(i).boundingRect;
            width = faceRectangle.width;
            height = faceRectangle.height;
            x = faceRectangle.x;
            y = faceRectangle.y;
            
            // creating a image with only the blob, from the grayscaleImage
            blobImg.allocate(width, height);
            blobImg = setPixelsSubRegion(grayscaleImage, x, y, width, height);
            
            // find the faces in the blob
            faceFinder.findHaarObjects(blobImg);
            
            // stores the number of faces detected in the blob
            blobNrFaces.push_back(faceFinder.blobs.size());
            
            // stores the information ablut the faces detected
            for (int i = 0; i < faceFinder.blobs.size(); i++) {
                blobFaceArray.push_back(faceFinder.blobs.at(i).boundingRect);
            }
            
            // Stores the blob image
            blobImgArray.push_back(blobImg);
            
            
            nrBlobs++;
        }
    }
    
}

//--------------------------------------------------------------
void ofApp::draw(){
    
    // Draws the Depth image
    depthImg.draw(0, 0, 400, 300);
    // Draws the Contour of the detected blobs
    contourFinder.draw(0, 0, 400, 300);
    // Draws the grayscale Image from the color camera
    grayscaleImage.draw(400, 0, 400, 300);
    
    // Drawing the blob images of the detected blobs
    int totalFaces = 0;
    int printWidth = 0;
    for (int i = 0; i < nrBlobs; i++) {
        blobImgArray[i].draw(printWidth+(i*10), 300);
        
        // Draws the detected faces from face finder
        ofNoFill();
        for(unsigned int j = 0; j < blobNrFaces[i]; j++) {
            ofRectangle cur = blobFaceArray[totalFaces];
            ofRect(cur.x + printWidth, cur.y + 300, cur.width, cur.height);
            totalFaces = totalFaces + 1;
        }
        
        printWidth = printWidth + blobImgArray.at(i).getWidth();
    }
    
    // Prints out the number of detected faces if there are any blobs detected. Prints out the FPS too.
    if (nrBlobs > 0) {
        ofSetHexColor(0xffffff);
        stringstream reportStr;
        reportStr << "Number of Faces Detected: "  <<  totalFaces << ", fps: " << ofGetFrameRate();
        
        ofDrawBitmapString(reportStr.str(), 20, 700);
    }
    
}

ofxCvGrayscaleImage ofApp::setPixelsSubRegion(ofxCvGrayscaleImage orgImage,int x, int y,int width, int height)
{
    ofxCvGrayscaleImage targetImg;
    bool color = false;
    unsigned char * pixels = orgImage.getPixels();
    int totalWidth = orgImage.getWidth();
    int subRegionLength = width * height;
    unsigned char subRegion[subRegionLength];
    
    int result_pix = 0;
    for (int i = y; i < y+height; i++)
    {
        for(int j = x; j < x+width; j++)
        {
            int base = (i * totalWidth) + j;
            subRegion[result_pix] = pixels[base];
            result_pix++;
        }
    }
    targetImg.setFromPixels(subRegion, width, height);
    return targetImg;
    
}


//--------------------------------------------------------------
void ofApp::exit() {
    kinect.setCameraTiltAngle(0); // zero the tilt on exit
    kinect.close();
    
}

//--------------------------------------------------------------
void ofApp::keyPressed(int key){
    
}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){
    
}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y){
    
}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){
    
}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button){
    
}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){
    
}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){
    
}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){
    
}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){
    
}